(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-standalone")

(define (ref-length lst) (length (cell-ref lst)))

(define (ref-length>0? lst) (> (ref-length lst) 0))

;;;;;;;;;;;;;;;;;;;;;

(define using-modules (make-cell (list-copy (cell-ref (global 'profile-modules)))))
(define using-list (global 'modules-list))

(define (get-modules)
  (let ((using (cell-ref using-modules)))
    (define-operation all-matches)
    (define-operation next-url-of)  
    (make-cell
     (filter (lambda(item) (not (memq item using)))
             (map next-url-of (all-matches (from-context 'atlas) '/acc-hook))))))

(define (get-name url)
  (define-operation params-of)
  (or
   (and-let* ((path ((from-context 'atlas) url))
              (params (params-of path)))
             (woo-get-option (cons #f params) 'acc-description #f))
   (sure-string url)))


(define avail-modules (get-modules))

(define active-if-modules (activity (ref-length>0? using-modules)))


;protection for modification buttons
(define (toggle-buttons)
  (add-button activity (ref-length>0? avail-modules))
  ((widgets del-button up-button dn-button)
   activity (ref-length>0? using-modules)))


(define (exchange-item from-list from-widget to-list to-widget)
  (let ((curr (from-widget current)))
    (cell-set! to-list (append1 (cell-ref to-list)
                                (list-ref (cell-ref from-list) curr)))
    (to-widget append-row (from-widget text)
               current (- (ref-length to-list) 1))

    (cell-set! from-list (list-remove (cell-ref from-list) curr))
    (from-widget remove curr)
    (and (ref-length>0? from-list) (from-widget current 0)))
  (toggle-buttons))

;add new profile item
(define (add-item)
  (exchange-item avail-modules avail-list
                 using-modules using-list))

;remove profile item
(define (del-item)
  (exchange-item using-modules using-list
                 avail-modules avail-list))


;move profile item down
(define (down-item)
  (let ((cur (using-list current)))
    (cell-set! using-modules (list-exchange (cell-ref using-modules) cur))
    (using-list rows (map get-name (cell-ref using-modules)))

    (if (< cur (- (using-list count) 1))
         (using-list current (+ 1 cur))
         (using-list current cur))))

(define (up-item)
  (let ((cur (using-list current)))
    (and (> cur 0)
         (begin
           (cell-set! using-modules (list-exchange (cell-ref using-modules) (- cur 1)))
           (using-list rows (map get-name (cell-ref using-modules)))
           (using-list current (- cur 1))))))


(define (save-profile)
  (woo-catch
   (lambda()    
     (let ((result (woo-first-command (woo-write (string-append "/profile/"
                                                                (or (cell-ref (global 'profile-short-name)) "autocreate"))
                                                 'acc-caption (full-name text)
                                                 'acc-modules (string-join (map sure-string (cell-ref using-modules)) ";")))))
       (or (cell-ref (global 'profile-short-name))
         (cell-set! (global 'profile-short-name) (woo-get-option result 'name))))
     #t)
   (lambda(reason) #f)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;

children-align center
(label (_ "Add/Remove/Move profile components"))

spacing 3
(hbox
  (layout-policy 100 -1)
  spacing 3
  (label (_ "Profile name:"))
  (document:id full-name
               (edit (or (cell-ref (global 'profile-full-name)) "")
                     (layout-policy -2 100)
                     (when changed
                       (apply-button activity (not-empty-string? (full-name text)))))))
                     

(hbox
  (layout-policy 100 -2)
  (vbox (layout-policy 30 100)
        (vertical-spacer)
        (document:id add-button (button "<<-" inactive
                                        tooltip (_ "Add new components to profile")
                                        (layout-policy 100 -1) (when clicked (add-item))))
	(document:id del-button (button "->>"
                                        tooltip (_ "Remove exising components from profile")
                                        active-if-modules (layout-policy 100 -1) (when clicked
                                                                                   (del-item))))
	(document:id up-button (button (_ "Up")
                                       tooltip (_" Move up profile components")
                                       active-if-modules (layout-policy 100 -1) (when clicked
                                                                                  (up-item))))
	(document:id dn-button (button (_ "Down")
                                       tooltip (_ "Move down profile components")
                                       active-if-modules (layout-policy 100 -1) (when clicked
                                                                                  (down-item))))
	(vertical-spacer))
  (document:id avail-list (listbox (layout-policy -2 100)
                                   (when selected (add-button activity #t))
                                   rows (map get-name (cell-ref avail-modules)) ) ))




(frame:buttons-view (button (_ "Quit") (when clicked (document:end)))
                    (document:id apply-button
                                 (button (_ "Apply")
                                         activity (not-empty-string?
                                                   (cell-ref (global 'profile-full-name)))
                                         (when clicked
                                          (and (save-profile)
                                               (begin
                                                 (cell-set! (global 'profile-modules)
                                                            (cell-ref using-modules))
                                                 (cell-set! (global 'profile-full-name)
                                                            (full-name text))))
                                          
                                          ((global 'visualize-profile)) )))
                    (button (_ "Discard") (when clicked
                                           ((global 'visualize-profile)) )))
