(document:surround "/std/frame")
(document:insert "/std/functions")
(document:insert "/standalone/profile")
(document:envelop with-translation _ "alterator-standalone")

width 600
height 560

;;load first module
(define (load-first)
  (acc-replace (or (cond-car (cell-ref profile-modules))
                   '/standalone/default) ))

;;setup widgets according loaded profile
(define (visualize-profile)
  ((widgets show-button edit-modules load-modules)
   activity #t)
  (modules-list rows (map read-description (cell-ref profile-modules))
                current 0)
  ((global 'document:popup) title (or (cell-ref profile-full-name)
                                      (_ "System management center")))
  (edit-modules visibility (not (cell-ref profile-system?)))

  ;;setup vibility
  (let ((>1 (length>1? (cell-ref profile-modules))))
    (show-button text (if >1 ">>" "<<"))
    (extra visibility >1
           current (if >1 select-tab help-tab)))
  (load-first))


(define current-module-number (make-cell #f))


(define confirm-message (_ "There are unsaved changes in the active module.
Do you want to apply the changes before running the new module
or discard the changes?"))

;; possible states:
;; locked - you cannot leave this module
;; need-confirm - need confirmation from user
;; apply - apply changes and go to the next module
;; discard - discard changes and go to the next module
;; cancel - stay on this module
(define (acc-process-change status)
  (case status
    ;;special values
    ((locked) (document:popup-information (_ "Module is locked") 'ok)
     (acc-process-change 'cancel))
    ((need-confirm) (acc-process-change (document:popup-warning confirm-message 'cancel 'apply 'discard)))
    ;;standard dialog return values
    ((cancel) (modules-list current (cell-ref current-module-number)) #t)
    ((apply) (if (frame:call-on-apply)
                        (acc-process-change 'discard)
                        (acc-process-change 'cancel)))
    ;;discard
    (else
     (acc-replace (list-ref (cell-ref profile-modules) (modules-list current))) )))

;;process module changing
(define (acc-replace . url)
  (cell-set! current-module-number (modules-list current))
  (apply frame:replace (append url
                               (list 'load-profile load-profile
                                     'visualize-profile visualize-profile))))


(define (make-extra)
  (document:id extra
               (tabbox
                (layout-policy 30 100)
                
                (document:id help-tab
                             (tab (_ "Help")
                                  (layout-policy 100 100)
                                  (help-place (layout-policy 100 100))))
                (document:id select-tab
                             (tab (_ "Modules")
                                  (layout-policy 100 100)
                                  children-align center
                                  (document:id modules-list
                                               (listbox
                                                (when selected
                                                  (and (load-modules activity)
                                                       (acc-process-change (frame:call-on-leave))))
                                                (layout-policy 100 -2)))
                                  (document:id edit-modules
                                               (button (_ "Edit profile")
                                                       (when clicked
                                                         ((widgets show-button
                                                                   load-modules
                                                                   edit-modules) inactive)
                                                         (acc-replace '/standalone/edit
                                                                      'modules-list
                                                                      modules-list
                                                                      'profile-modules
                                                                      profile-modules
                                                                      'profile-full-name
                                                                      profile-full-name
                                                                      'profile-short-name
                                                                      profile-short-name))))
                                  (document:id load-modules
                                               (button (_ "Load profile")
                                                       (when clicked
                                                         ((widgets show-button
                                                                   load-modules
                                                                   edit-modules) inactive)
                                                         (acc-replace '/standalone/select)))))))))

(define (make-view)
  (vbox
   (layout-policy -2 100)
   (groupbox ""
             (layout-policy 100 -2)
             (frame:make-page))
   (hbox (layout-policy 100 -1)
         (document:id show-button
                      (button "||"
                              (tooltip (_ "Press here to show/hide help or profile components")
                                       "help.png")
                              (when clicked (extra visibility (not (extra visibility)))
                                    (show-button text (if (extra visibility) ">>" "<<")))))
         (frame:make-buttons-view (layout-policy -2 100) ))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;setup default globals
(frame:clear-on-leave)
(frame:clear-on-apply)
(frame:clear-buttons-view)

children-align center

(load-profile (global 'profile-name))

(hbox
 (layout-policy 99 99)
 (make-extra)
 (make-view))

(visualize-profile)

;;first time simulation
(document:root (when loaded (page-id loaded)))
